package pt.uc.dei.wsdbench.sut.service.delegate.impl.pedro;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.List;
import java.util.concurrent.LinkedBlockingQueue;

public class Database {

    private final String CONNTYPETHIN = "thin";
    private String server = null;
    private int port = -1;
    private String sid = null;
    private String url = null;
    private String userName = null;
    private String passwd = null;
    private String connType = this.CONNTYPETHIN;
    private String driverName = "oracle.jdbc.driver.OracleDriver";
    private static LinkedBlockingQueue<Connection> conns;
    private Connection con;
    private Statement stmt;
    private ResultSet resultSet = null;
    private int fetchSize = -1;

    public Database(String server, int port, String sid, String userName,
            String passwd) {
        this.server = server;
        this.port = port;
        this.sid = sid;
        this.userName = userName;
        this.passwd = passwd;
        buildConns();
    }

    private void buildConns() {
        synchronized (Database.class) {
            if (conns == null) {
                conns = new LinkedBlockingQueue<Connection>();
                for (int i = 1; i < 20; i++) {
                    try {
                        Class.forName(this.driverName);
                        this.url = this.getUrl();// constructs the URL
                        Connection c = DriverManager.getConnection(this.url,
                                this.userName, this.passwd);
                        if (c == null) {
                            System.out.println("C NULL");
                        } else {
                            System.out.println("Connection Build " + i);
                        }
                        conns.put(c);
                    } catch (Exception e) {
                        System.out.println("Cannot Connect: " + i);
                    }
                }
            }
        }
    }

    public boolean connectDB() {
        while (true) {
            try {
                this.con = conns.take();
                if (con != null) {
                    return true;
                }
            } catch (InterruptedException e) {
            }
        }
    }

    public String closeDB() {
        if (this.con != null) {
            try {
                con.rollback();
            } catch (Exception e) {
                System.out.println("Not .... Rollbacked...");
            }
        }
        if (this.stmt != null) {
            try {
                stmt.close();
            } catch (Exception e) {
                System.out.println("Statement NOK...");
            }
        }
        if (this.con != null) {
            try {
                conns.put(con);
                con = null;
            } catch (InterruptedException e) {
            }
        }
        return null;
    }

    public void setFetchSize(int fs) {
        fetchSize = fs;
    }

    public ResultSet executeQuery(String query) throws SQLException {

        if (this.con != null) {
            stmt = con.createStatement();
            if (fetchSize != -1) {
                stmt.setFetchSize(fetchSize);
            }
            try {
                resultSet = stmt.executeQuery(query);
            } catch (SQLException e) {
                this.closeDB();
                throw e;
            }
        } else {
            throw new SQLException("Null Connection");
        }
        return resultSet;
    }

    public ResultSet executeQuery(String query, List<Long> list)
            throws SQLException {

        PreparedStatement statement = null;
        if (this.con != null) {
            statement = this.con.prepareStatement(query);
            int tam = list.size();
            for (int i = 0; i < tam; ++i) {
                Long l = list.get(i);
                statement.setInt(1, l.intValue());
            // ResultSet rs = statement.executeQuery();
            // Rs.close();
            }

            try {
                resultSet = statement.executeQuery();
            } catch (SQLException e) {
                this.closeDB();
                throw e;
            }

        // resultSet = stmt.executeQuery(query);
        } else {
            throw new SQLException("Null Connection");
        }
        return resultSet;
    }

    public int executeCmd(String cmd) throws SQLException {
        int rc = 0;
        if (this.con != null) {
            try {
                stmt = con.createStatement();
                rc = stmt.executeUpdate(cmd);
            } catch (SQLException e) {
                this.closeDB();
                throw e;
            }
        } else {
            throw new SQLException("Null Connection");
        }
        return rc;
    }

    public String getUrl() {
        return ("jdbc:oracle:" + this.connType + ":@" + this.server + ":" + this.port + ":" + this.sid);
    }
}
